from model import Model


def create(idea, title):
    # Move the logic you want to be available as a library here
    # This function should be a generator that yields results based on the given idea and title

    """for data in generate_response(idea, title):  # assuming generate_response is the function you want to use
        yield data"""
    ...